#import "DiningMeal.h"
#import "DiningDay.h"
#import "DiningMealItem.h"
#import "DiningDietaryFlag.h"
#import "DiningData.h"
#import "CoreDataManager.h"
#import "Foundation+MITAdditions.h"

@implementation DiningMeal

@dynamic name;
@dynamic startTime;
@dynamic endTime;
@dynamic message;
@dynamic day;
@dynamic items;

NSString * const MITDiningMealTimeFormatterKey = @"DiningMeal.MITDiningMealTimeFormatter";
+ (DiningMeal *)newMealWithDictionary:(NSDictionary* )dict {
    // Meals need to have a name and either a message or both a start and end time.
    if (!dict[@"name"] || !((dict[@"start_time"] && dict[@"end_time"]) || dict[@"message"])) {
        return nil;
    }
    
    DiningMeal *meal = [CoreDataManager insertNewObjectForEntityForName:@"DiningMeal"];
    
    if (dict[@"name"]) {
        meal.name = [dict[@"name"] lowercaseString];
    }
    
    if (dict[@"message"]) {
        meal.message = dict[@"message"];
    }

    
    NSDateFormatter *timeFormatter = nil;
    {
        NSMutableDictionary *threadDictionary = [[NSThread currentThread] threadDictionary];
        timeFormatter = threadDictionary[MITDiningMealTimeFormatterKey];
        if (!timeFormatter) {
            timeFormatter = [[NSDateFormatter alloc] init];
            [timeFormatter setDateFormat:@"HH:mm:ss"];
            threadDictionary[MITDiningMealTimeFormatterKey] = timeFormatter;
        }
    }
    

    if (dict[@"start_time"]) {
        NSDate *date = [timeFormatter dateFromString:dict[@"start_time"]];
        meal.startTime = date;
    }

    if (dict[@"end_time"]) {
        NSDate *date = [timeFormatter dateFromString:dict[@"end_time"]];
        meal.endTime = date;
    }
    
    NSArray *allDietaryFlags = [DiningData sharedData].allFlags;
    
    NSInteger i = 0;
    for (NSDictionary *itemDict in dict[@"items"]) {
        DiningMealItem *item = [DiningMealItem newItemWithDictionary:itemDict possibleFlags:allDietaryFlags];
        item.ordinality = @(i);
        i++;
        [meal addItemsObject:item];
    }

    return meal;
}

// There appears to be a bug in Apple's autogenerated NSOrderedSet accessors: http://stackoverflow.com/questions/7385439/exception-thrown-in-nsorderedset-generated-accessors
- (void)addItemsObject:(DiningMeal *)value {
    NSMutableOrderedSet* tempSet = [NSMutableOrderedSet orderedSetWithOrderedSet:self.items];
    [tempSet addObject:value];
    self.items = tempSet;
}

/**
 
 Could be nil. That happens when there's no times or message. Views can decide how they want to display that. Generally, if a meal isn't offered then it won't even be listed, so something weirder than just "Closed" must be going on if this returns nil.
 */

- (NSString *)hoursSummary {
    
    if (!self.startTime || !self.endTime) {
        return self.message;
    }
    
    NSString *startString = [self.startTime MITShortTimeOfDayString];
    NSString *endString = [self.endTime MITShortTimeOfDayString];
    
    return [[NSString stringWithFormat:@"%@ - %@", startString, endString] lowercaseString];
}

@end
